<?php

namespace FiloBlu\StockaFix\Ui\Component\Listing;

/**
 * Class Columns
 * @package FiloBlu\StockaFix\Ui\Component\Listing
 */
class Columns extends \Magento\Ui\Component\Listing\Columns
{
    /**
     * @var int
     */
    const DEFAULT_COLUMNS_MAX_ORDER = 3;

    /**
     * @var \Magento\Framework\App\RequestInterface
     */
    protected $request;

    /**
     * @var \Magento\Framework\View\Element\UiComponentFactory
     */
    protected $uiComponentFactory;
    /**
     * @var \Magento\Framework\App\ResourceConnection
     */
    protected $resourceConnection;

    /**
     * @var \FiloBlu\StockaFix\Model\ResourceModel\Report\CollectionFactory
     */
    protected $reportCollectionFactory;

    /**
     * @var \FiloBlu\StockaFix\Helper\ReportHelper
     */
    protected $reportHelper;

    /**
     * Columns constructor.
     * @param \Magento\Framework\View\Element\UiComponent\ContextInterface $context
     * @param \Magento\Framework\App\RequestInterface $request
     * @param \Magento\Framework\View\Element\UiComponentFactory $uiComponentFactory
     * @param \Magento\Framework\App\ResourceConnection $resourceConnection
     * @param \FiloBlu\StockaFix\Model\ResourceModel\Report\CollectionFactory $reportCollectionFactory
     * @param \FiloBlu\StockaFix\Helper\ReportHelper $reportHelper
     * @param array $components
     * @param array $data
     */
    public function __construct(
        \Magento\Framework\View\Element\UiComponent\ContextInterface $context,
        \Magento\Framework\App\RequestInterface $request,
        \Magento\Framework\View\Element\UiComponentFactory $uiComponentFactory,
        \Magento\Framework\App\ResourceConnection $resourceConnection,
        \FiloBlu\StockaFix\Model\ResourceModel\Report\CollectionFactory $reportCollectionFactory,
        \FiloBlu\StockaFix\Helper\ReportHelper $reportHelper,
        array $components = [],
        array $data = []
    )
    {
        $this->uiComponentFactory = $uiComponentFactory;
        $this->request = $request;
        $this->resourceConnection = $resourceConnection;
        $this->reportCollectionFactory = $reportCollectionFactory;
        $this->reportHelper = $reportHelper;
        parent::__construct($context, $components, $data);
    }

    /**
     * {@inheritdoc}
     */
    public function prepare()
    {
        $columnSortOrder = self::DEFAULT_COLUMNS_MAX_ORDER;

        $id = $this->request->getParam('id');

        /** @var \FiloBlu\StockaFix\Model\Report $report */

        $report = $this->reportCollectionFactory->create()->addFieldToFilter('report_id', ['eq', $id])->getFirstItem();

        $meta = $this->reportHelper->getReportMeta($report);

        foreach ($meta as $attribute => $info) {
            if (!isset($this->components[$attribute])) {
                $arguments = [
                    'data' => [
                        'config' =>  $this->getColumnFromMeta($info, $attribute, $columnSortOrder++)
                    ],
                    'context' => $this->context
                ];
                $column = $this->uiComponentFactory->create($attribute, 'column', $arguments);
                $column->prepare();
                $this->addComponent($attribute, $column);
            }
        }
        parent::prepare();
    }

    /**
     * @param $column
     * @param $attribute
     * @param $sortOrder
     * @return array
     */
    protected function getColumnFromMeta($column, $attribute, $sortOrder)
    {

        $type = $column['Type'];
        $label = ucfirst(str_replace('_', ' ', $attribute));

        switch (true) {

            /* Numeric */
            case (stripos($type, 'int', 0) === 0):

                return [
                    'label' => $label,
//                    'dataType' => 'int',
                    'add_field' => false,
                    'visible' => 1,
                    'filter' => 'textRange',
                    'sortOrder' => $sortOrder,
                    'component' => 'Magento_Ui/js/grid/columns/column'
                ];
                /* Date */
            case (stripos($type, 'date', 0) === 0):
                [
                    'label' => $label,
                    'dataType' => 'date',
                    'add_field' => false,
                    'visible' => 1,
                    'filter' => 'dateRange',
                    'sortOrder' => $sortOrder,
                    'component' => 'Magento_Ui/js/grid/columns/date'
                ];


                return;
            /* String */
            case (stripos($type, 'varchar', 0) === 0):
            default:

                return [
                    'label' => $label,
                    'dataType' => 'text',
                    'add_field' => false,
                    'visible' => 1,
                    'filter' => 'text',
                    'sortOrder' => $sortOrder,
                    'component' => 'Magento_Ui/js/grid/columns/column'
                ];
        }

    }

}
